-- Module
local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local Bag = BaikItemLevel:NewModule("Bag", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")
local ItemLevelFrame = BaikItemLevel.ItemLevelFrame
local ItemHelper = BaikItemLevel.ItemHelper
local TableHelper = BaikItemLevel.TableHelper

-- Constants
local MAX_BAGS = 12

-- Helper Functions
local function GetBagFrame(containerId)
    return _G[string.format("ContainerFrame%d", containerId + 1)]
end

local function GenerateSlotFrames(containerId)
    local frames = {}
    local size = GetContainerNumSlots(containerId)
    for idx=1,size,1 do
        frames[idx] = _G[string.format("ContainerFrame%dItem%d", containerId + 1, size - idx + 1)]
    end

    return frames
end

local function GenerateBankSlots()
    local frames = {}
    local size = GetContainerNumSlots(BANK_CONTAINER)
    for idx=1,size,1 do
        frames[idx] = _G[string.format("BankFrameItem%d", idx)]
    end

    return frames
end

local function GetItemLevel(container, slot)
    local itemLink = GetContainerItemLink(container, slot);

    return ItemHelper:GetItemLevel(itemLink)
end

-- Bag API
function Bag:CreateBags()
    local bags = {}
    for id=0,MAX_BAGS,1 do
        local frame = GetBagFrame(id)
        frame.baikId = id
        self:HookScript(frame, "OnShow", "Load")
    end
    self:HookScript(BankFrame, "OnShow", "LoadBank")

    return bags
end

function Bag:Load(frame)
    if self:IsHooked(frame, "OnShow") then
        self:Unhook(frame, "OnShow")
        local name = string.format("BaikBag%dFrame", frame.baikId)
        self.bags[frame] = ItemLevelFrame.new(name, frame, GenerateSlotFrames(frame.baikId),
                                         function(slot) return GetItemLevel(frame.baikId, slot) end)
    end

    local bag = self.bags[frame]
    bag:Update()
    if bag:HasMissing() then
        self.missingBags[bag] = true
        if not self.callback then
            self.callback = self:ScheduleTimer("LoadMissing", 5)
        end
    end
end

function Bag:LoadBank()
    if self:IsHooked(BankFrame, "OnShow") then
        self:Unhook(BankFrame, "OnShow")
        self.bags[BankFrame] = ItemLevelFrame.new("BaikBankFrame", BankFrame, GenerateBankSlots(),
                                         function(slot) return GetItemLevel(BANK_CONTAINER, slot) end)
    end

    local bag = self.bags[BankFrame]
    bag:Update()
    if bag:HasMissing() then
        self.missingBags[bag] = true
        if not self.callback then
            self.callback = self:ScheduleTimer("LoadMissing", 5)
        end
    end
end

function Bag:LoadMissing()
    local bagList = TableHelper:Copy(self.missingBags)
    self.missingBags = {}
    self.callback = nil
    for bag, _ in pairs(bagList) do
        bag:UpdateMissing()
        if bag:HasMissing() then
            self.missingBags[bag] = true
            if not self.callback then
                self.callback = self:ScheduleTimer("LoadMissing", 5)
            end
        end
    end
end

function Bag:OnUpdate()
    for _, bag in pairs(self.bags) do
        bag:Update()
    end
end

-- Event Callbacks
function Bag:PLAYER_AVG_ITEM_LEVEL_UPDATE()
    for _, bag in pairs(self.bags) do
        bag:UpdateColor()
    end
end

function Bag:BAG_UPDATE(opt, id)
    if id == BANK_CONTAINER then
        self.updateBags[id] = self.bags[BankFrame]
    else
        local frame = GetBagFrame(id)
        self.updateBags[id] = self.bags[frame]
    end
end

function Bag:BAG_UPDATE_DELAYED()
    local bagCache = TableHelper:Copy(self.updateBags)
    self.updateBags = {}

    for id, bag in pairs(bagCache) do
        if bag:Size() ~= GetContainerNumSlots(id) then
            bag:SetTable(GenerateSlotFrames(id))
        end
        bag:Update()
    end
end

function Bag:PLAYERBANKSLOTS_CHANGED()
    local bankBag = self.bags[BankFrame]
    bankBag:Update()
end

-- Ace3 Callbacks
function Bag:OnInitialize()
    self.updateBags = {}
    self.missingBags = {}
    BaikItemLevel:DPrint("Bag OnInitialize")
end

function Bag:OnEnable()
    self.bags = self:CreateBags()
    self:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE")
    self:RegisterEvent("BAG_UPDATE")
    self:RegisterEvent("BAG_UPDATE_DELAYED")
    self:RegisterEvent("PLAYERBANKSLOTS_CHANGED")
    BaikItemLevel:DPrint("Bag OnEnable")
end

function Bag:OnDisable()
    BaikItemLevel:DPrint("Bag OnDisable")
end
