local BaikItemLevel = LibStub("AceAddon-3.0"):GetAddon("BaikItemLevel")
local TableHelper = BaikItemLevel.TableHelper
local ItemLevelBadge = BaikItemLevel.ItemLevelBadge
local ItemLevelFrame = {}

-- Add Module to Addon
ItemLevelFrame.__index = ItemLevelFrame
BaikItemLevel.ItemLevelFrame = ItemLevelFrame

-- ItemLevelFrame API
function ItemLevelFrame.new(frameName, parentFrame, slotTable, getItemLevel)
    assert(frameName ~= nil, "frameName is nil")
    assert(parentFrame ~= nil, "parentFrame is nil")
    assert(slotTable ~= nil, "slotTable is nil")
    assert(getItemLevel ~= nil, "getItemLevel is nil")
    local self = setmetatable({}, ItemLevelFrame)

    -- Assign variables
    self.frame = frame
    self.frameName = frameName
    self.badges = {}
    self.missing = {}
    self.slotTable = slotTable
    self.getItemLevel = getItemLevel

    return self
end

function ItemLevelFrame:SetSlot(slot)
    assert(slot ~= nil, "slot is nil")
    assert(self.slotTable[slot] ~= nil, "slotFrame is nil")
    local itemLevel, hasItem, quality = self.getItemLevel(slot)
    local badge = self.badges[slot]

    -- Checks if slot has item
    if hasItem then
        -- Checks to see if we can't get itemLevel information
        if not itemLevel then
            table.insert(self.missing, slot)
            itemLevel = ItemLevelBadge.INVALID
        end

        -- Creates new badge object
        if not badge then
            badge = ItemLevelBadge.new(string.format("%s_%d", self.frameName, slot), self.slotTable[slot])
            self.badges[slot] = badge
        end

        -- Updates badge itemLevel
        badge:SetLevel(itemLevel, quality)
    elseif badge then
        -- Item no longer exists
        badge:Hide()
        self.badges[slot] = nil
    end
end

function ItemLevelFrame:UpdateColor()
    for _, badge in pairs(self.badges) do
        badge:UpdateColor()
    end
end

function ItemLevelFrame:UpdateMissing()
    local missingCache = TableHelper:Copy(self.missing)
    self.missing = {}
    for _, slot in pairs(missingCache) do
        self:SetSlot(slot)
    end
end

function ItemLevelFrame:HasMissing()
    return not TableHelper:IsEmpty(self.missing)
end

function ItemLevelFrame:ClearBadges()
    for _, badge in pairs(self.badges) do
        badge:Hide()
    end
    self.badges = {}
end

function ItemLevelFrame:SetTable(slotTable)
    assert(slotTable ~= nil, "slotTable is nil")
    self:ClearBadges()
    self.slotTable = slotTable
    self:Update()
end

function ItemLevelFrame:Size()
    return TableHelper:Size(self.slotTable)
end

function ItemLevelFrame:Update()
    for slot, _ in pairs(self.slotTable) do
        self:SetSlot(slot)
    end
end
